@@ -1,13 +1,38 @@
-#!D:/bld/uhd_1636999620862/_h_env/python.exe
+#!D:/bld/uhd_1642002131301/_h_env/python.exe
 #
 # Copyright 2018 Ettus Research, a National Instruments Company
+# Copyright 2020 Ettus Research, a National Instruments Brand
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 #
 """
-Download image files required for USRPs
+Download image files required for USRPs.
+
+Usage: The `uhd_images_downloader` should work, "out of the box", with no command line arguments.
+Assuming your computer has an internet connection to [files.ettus.com], simply run the utility
+every time you update UHD, and the installed images for your devices should always be up to date.
+
+
+Images will be downloaded on a per-target basis. That is, there are image packages for a desired
+device and configuration. Users can specify which image packages they would plan to use. To see a
+list of available targets, run `uhd_images_downloader --list-targets`. The left column of the
+printout will be a list of available image archives. From there, you can construct a regular
+expression which matches to the targets you wish to download. For example, in order to download all
+image packages related to the X300 product line, users may run
+`uhd_images_downloader --types x3.*`.
+
+
+The `uhd_images_downloader` uses a manifest to look-up the URLs of image packages to download.
+Downloaded images are recorded in an inventory file that lives in the images install location.
+This allows the downloader to skip images that were previously downloaded, and haven't changed
+since.
+
+Manifests are built into the downloader, but can also be accessed at uhd/images/manifest.txt.
+
+Inventory files are JSON files called `inventory.json`, by default. It is possible to specify the
+inventory file in command line arguments, but we don't recommend using this functionality unless
+you're really sure you need it.
 """
-from __future__ import print_function
 import argparse
 import hashlib
 import json
@@ -19,26 +44,10 @@ import sys
 import tempfile
 import zipfile
 import platform
+from urllib.parse import urljoin  # Python 3
+
 # For all the non-core-library imports, we will be extra paranoid and be very
 # nice with error messages so that everyone understands what's up.
-try:
-    from urllib.parse import urljoin  # Python 3
-except ImportError:
-    from urlparse import urljoin      # Python 2
-try:
-    from builtins import input
-except ImportError:
-    input = raw_input
-try:
-    from six import iteritems
-except ImportError:
-    sys.stdout.write(
-        "[ERROR] Missing module 'six'! Please install it, e.g., by "
-        "running 'pip install six' or any other tool that can install "
-        "Python modules.\n")
-    if platform.system() == 'Windows':
-        input('Hit Enter to continue.')
-    exit(0)
 try:
     import requests
 except ImportError:
@@ -50,28 +59,32 @@ except ImportError:
         input('Hit Enter to continue.')
     exit(0)
 
-
-
+# pylint: disable=bad-whitespace
+_USERNAME_VARIABLE        = "UHD_IMAGES_USER"
+_PASSWORD_VARIABLE        = "UHD_IMAGES_PASSWORD"
 _DEFAULT_TARGET_REGEX     = "(fpga|fw|windrv)_default"
 _BASE_DIR_STRUCTURE_PARTS = ["share", "uhd", "images"]
-_DEFAULT_INSTALL_PATH     = os.path.join("D:/bld/uhd_1636999620862/_h_env/Library", *_BASE_DIR_STRUCTURE_PARTS)
-_DEFAULT_BASE_URL         = "http://files.ettus.com/binaries/cache/"
+_DEFAULT_INSTALL_PATH     = os.path.join("D:/bld/uhd_1642002131301/_h_env/Library", *_BASE_DIR_STRUCTURE_PARTS)
+_DEFAULT_BASE_URL         = "https://files.ettus.com/binaries/cache/"
 _INVENTORY_FILENAME       = "inventory.json"
 _CONTACT                  = "support@ettus.com"
 _DEFAULT_BUFFER_SIZE      = 8192
 _DEFAULT_DOWNLOAD_LIMIT   = 100 * 1024 * 1024 # Bytes
 _ARCHIVE_DEFAULT_TYPE     = "zip"
-_UHD_VERSION              = "3.15.0.0-release"
-# Note: _MANIFEST_CONTENTS are placed at the bottom of this file for aesthetic reasons
-_LOG_LEVELS = {"TRACE": 1,
-               "DEBUG": 2,
-               "INFO": 3,
-               "WARN": 4,
-               "ERROR": 5}
+_UHD_VERSION              = "4.1.0.5-release"
+_LOG_LEVELS = {
+    "TRACE": 1,
+    "DEBUG": 2,
+    "INFO":  3,
+    "WARN":  4,
+    "ERROR": 5,
+}
 _LOG_LEVEL = _LOG_LEVELS["INFO"]
 _YES = False
 _PROXIES = {}
-
+_BASE_URL_ENV_VAR = "UHD_IMAGES_URL"
+# pylint: enable=bad-whitespace
+# Note: _MANIFEST_CONTENTS are placed at the bottom of this file for aesthetic reasons
 
 def log(level, message):
     """Logging function"""
@@ -86,15 +99,15 @@ def parse_args():
     Setup argument parser and parse. Also does some sanity checks and sets some
     global variables we want to use.
     """
-    parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
+    parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter,
+                                     description=__doc__)
     parser.add_argument('-t', '--types', action='append',
                         help="RegEx to select image sets from the manifest file.")
-    parser.add_argument('-i', '--install-location',
-                        default=None,
+    parser.add_argument('-i', '--install-location', type=str, default=None,
                         help="Set custom install location for images")
-    parser.add_argument('-m', '--manifest-location', type=str, default="",
+    parser.add_argument('-m', '--manifest-location', type=str, default=None,
                         help="Set custom location for the manifest file")
-    parser.add_argument('-I', '--inventory-location', type=str, default="",
+    parser.add_argument('-I', '--inventory-location', type=str, default=None,
                         help="Set custom location for the inventory file")
     parser.add_argument('-l', '--list-targets', action="store_true", default=False,
                         help="Print targets in the manifest file to stdout, and exit.\n"
@@ -108,12 +121,14 @@ def parse_args():
                              "larger than this will require approval, either "
                              "interactively, or by providing --yes.")
     parser.add_argument("--http-proxy", type=str,
-                        help="Specify HTTP proxy in the format "
-                        "http://user:pass@1.2.3.4:port\n"
+                        help="Specify HTTP(S) proxy in the format "
+                        "http[s]://user:pass@1.2.3.4:port\n"
                         "If this this option is not given, the environment "
-                        "variable HTTP_PROXY can also be used to specify a proxy.")
-    parser.add_argument("-b", "--base-url", type=str, default=_DEFAULT_BASE_URL,
-                        help="Set base URL for images download location")
+                        "variables HTTP_PROXY/HTTPS_PROXY can also be used to specify a proxy.")
+    parser.add_argument("-b", "--base-url", type=str,
+                        help="Set base URL for images download location. "
+                        "Defaults to ${} if set, or {} otherwise.".format(
+                            _BASE_URL_ENV_VAR, _DEFAULT_BASE_URL))
     parser.add_argument("-k", "--keep", action="store_true", default=False,
                         help="Keep the downloaded images archives in the image directory")
     parser.add_argument("-T", "--test", action="store_true", default=False,
@@ -129,8 +144,18 @@ def parse_args():
                         help="Decrease verbosity level")
     parser.add_argument('-v', '--verbose', action='count', default=0,
                         help="Increase verbosity level")
-    # Some sanitation that's easier to handle outside of the argparse framework:
     args = parser.parse_args()
+    # Set the verbosity
+    global _LOG_LEVEL
+    log("TRACE", "Default log level: {}".format(_LOG_LEVEL))
+    _LOG_LEVEL = _LOG_LEVEL - args.verbose + args.quiet
+    # Some sanitation that's easier to handle outside of the argparse framework:
+    if not args.base_url:
+        if os.environ.get(_BASE_URL_ENV_VAR):
+            args.base_url = os.environ.get(_BASE_URL_ENV_VAR)
+        else:
+            args.base_url = _DEFAULT_BASE_URL
+    log("INFO", "Using base URL: {}".format(args.base_url))
     if not args.base_url.endswith('/') and args.base_url != "":
         args.base_url += '/'
     if args.yes:
@@ -139,10 +164,7 @@ def parse_args():
     if args.http_proxy:
         global _PROXIES
         _PROXIES['http'] = args.http_proxy
-    # Set the verbosity
-    global _LOG_LEVEL
-    log("TRACE", "Default log level: {}".format(_LOG_LEVEL))
-    _LOG_LEVEL = _LOG_LEVEL - args.verbose + args.quiet
+        _PROXIES['https'] = args.http_proxy
     return args
 
 
@@ -160,7 +182,7 @@ def get_images_dir(args):
     return _DEFAULT_INSTALL_PATH
 
 
-def ask_permission(question, default_no=True):
+def ask_permission(question):
     """
     Ask the question, and have the user type y or n on the keyboard. If the
     global variable _YES is true, this always returns True without asking the
@@ -171,8 +193,7 @@ def ask_permission(question, default_no=True):
         log("DEBUG", "Assuming the answer is 'yes' for this question: " +
             question)
         return True
-    postfix = "[y/N]" if default_no else "[Y/n]"
-    answer = input(question + " " + postfix)
+    answer = input(question + " [y/N]")
     if answer and answer[0].lower() == 'y':
         return True
     return False
@@ -204,11 +225,16 @@ def get_manifest_raw(args):
     needs to be parsed to be of any practical use.
     """
     # If we're given a path to a manifest file, use it
-    if os.path.exists(args.manifest_location):
+    if args.manifest_location:
         manifest_fn = args.manifest_location
-        log("INFO", "Using manifest file at location: {}".format(manifest_fn))
-        with open(manifest_fn, 'r') as manifest_file:
-            manifest_raw = manifest_file.read()
+        if os.path.exists(manifest_fn):
+            log("INFO", "Using manifest file at location: {}".format(manifest_fn))
+            with open(manifest_fn, 'r') as manifest_file:
+                manifest_raw = manifest_file.read()
+        else:
+            log("WARN", "Invalid manifest file specified: {}, using default"
+                .format(manifest_fn))
+            manifest_raw = _MANIFEST_CONTENTS       
     # Otherwise, use the CMake Magic manifest
     else:
         manifest_raw = _MANIFEST_CONTENTS
@@ -316,10 +342,35 @@ def print_target_list(manifest, args):
                 base=args.base_url,
                 relpath=value["url"]))
     else:
-        for manifest_item in iteritems(manifest):
+        for manifest_item in manifest.items():
             print(args.base_url+manifest_item[1]["url"])
 
 
+def parse_auth(env):
+    """
+    Parse a (potentially None) environment variable string as a username:password
+    for consumption by requests.
+    """
+    username = env.get(_USERNAME_VARIABLE, None)
+    password = env.get(_PASSWORD_VARIABLE, None)
+    if not username and not password:
+        return None
+
+    if not username:
+        raise RuntimeError(
+            "Password variable {} was specified, so username variable {} must be set as well"
+            .format(_PASSWORD_VARIABLE, _USERNAME_VARIABLE)
+        )
+
+    if not password:
+        raise RuntimeError(
+            "Username variable {} was specified, so password variable {} must as well"
+            .format(_USERNAME_VARIABLE, _PASSWORD_VARIABLE)
+        )
+
+    return (username, password)
+
+
 def download(
         images_url,
         filename,
@@ -330,29 +381,37 @@ def download(
     """ Run the download, show progress """
     download_limit = download_limit or _DEFAULT_DOWNLOAD_LIMIT
     log("TRACE", "Downloading {} to {}".format(images_url, filename))
+    auth = parse_auth(os.environ)
     try:
         resp = requests.get(images_url, stream=True, proxies=_PROXIES,
-                            headers={'User-Agent': 'UHD Images Downloader'})
+                            headers={'User-Agent': 'UHD Images Downloader'},
+                            auth=auth)
     except TypeError:
         # requests library versions pre-4c3b9df6091b65d8c72763222bd5fdefb7231149
         # (Dec.'12) workaround
         resp = requests.get(images_url, prefetch=False, proxies=_PROXIES,
-                            headers={'User-Agent': 'UHD Images Downloader'})
+                            headers={'User-Agent': 'UHD Images Downloader'},
+                            allow_redirects=True, auth=auth)
     if resp.status_code != 200:
         raise RuntimeError("URL does not exist: {}".format(images_url))
-    filesize = float(resp.headers['content-length'])
+    filesize = float(resp.headers.get('content-length', -1))
     if filesize > download_limit:
         if not ask_permission(
                 "The file size for this target ({:.1f} MiB) exceeds the "
                 "download limit ({:.1f} MiB). Continue downloading?".format(
                     filesize/1024**2, download_limit/1024**2)):
             return 0, 0, ""
+    if filesize == -1:
+        if not ask_permission(
+                "The file size for this target could not be determined. "
+                "Continue downloading?"):
+            return 0, 0, ""
     filesize_dl = 0
     base_filename = os.path.basename(filename)
     if print_progress and not sys.stdout.isatty():
         print_progress = False
         log("INFO", "Downloading {}, total size: {} kB".format(
-            base_filename, filesize/1000))
+            base_filename, filesize/1000 if filesize > 0 else "-unknown-"))
     with open(filename, "wb") as temp_file:
         sha256_sum = hashlib.sha256()
         for buff in resp.iter_content(chunk_size=buffer_size):
@@ -361,10 +420,13 @@ def download(
                 filesize_dl += len(buff)
                 sha256_sum.update(buff)
             if print_progress:
-                status = r"%05d kB / %05d kB (%03d%%) %s" % (
-                    int(math.ceil(filesize_dl / 1000.)), int(math.ceil(filesize / 1000.)),
-                    int(math.ceil(filesize_dl * 100.) / filesize),
-                    base_filename)
+                status = r"%05d kB " % int(math.ceil(filesize_dl / 1000.))
+                if filesize > 0:
+                    status += r"/ %05d kB (%03d%%) " % (
+                        int(math.ceil(filesize / 1000.)),
+                        int(math.ceil(filesize_dl * 100.) / filesize),
+                    )
+                status += base_filename
                 if os.name == "nt":
                     status += chr(8) * (len(status) + 1)
                 else:
@@ -373,6 +435,8 @@ def download(
                 sys.stdout.flush()
     if print_progress:
         print('')
+    if filesize <= 0:
+        filesize = filesize_dl
     return filesize, filesize_dl, sha256_sum.hexdigest()
 
 
@@ -507,11 +571,30 @@ def main():
             "\n".join("{}".format(item) for item in manifest.items())
         ))
 
+        # Strip down path until we find first parent directory of images_dir
+        # that exists and then check if we have proper write permissions
+        images_dir_parent = images_dir
+        while not os.path.exists(os.path.abspath(images_dir_parent)):
+            path_pair = os.path.split(images_dir_parent)
+            if images_dir_parent == path_pair[0]:
+                # Prevent infinite loop
+                break
+            images_dir_parent = path_pair[0]
+        if not os.access(os.path.abspath(images_dir_parent), os.W_OK):
+            log("ERROR", "Invalid permissions to write images destination")
+            return False
+
         # Read the inventory into a dictionary we can perform lookups on
-        if os.path.isfile(args.inventory_location):
-            inventory_fn = args.inventory_location
+        default_inventory_fn = os.path.join(images_dir, _INVENTORY_FILENAME)
+        if args.inventory_location:
+            if os.path.isfile(args.inventory_location):
+                inventory_fn = args.inventory_location
+            else:
+                log("WARN", "Invalid inentory file specified: {}, using default: {}"
+                    .format(args.inventory_location, default_inventory_fn))
+                inventory_fn = default_inventory_fn
         else:
-            inventory_fn = os.path.join(images_dir, _INVENTORY_FILENAME)
+            inventory_fn = default_inventory_fn
         inventory = parse_inventory(inventory_fn=inventory_fn)
         log("TRACE", "Inventory: {}\n{}".format(
             os.path.abspath(inventory_fn),
@@ -571,49 +654,59 @@ def main():
 # Placing this near the end of the file so we don't clutter the top
 _MANIFEST_CONTENTS = """# UHD Image Manifest File
 # Target    hash    url     SHA256
+
+# X410-Series
+x4xx_x410_fpga_default          uhd-26793b8       x4xx/uhd-26793b8/x4xx_x410_fpga_default-g26793b8.zip       3093bb84f039d831253d17b43806ac00136af60e1142f47bcb65b7589f51ad79
+x4xx_x410_cpld_default          uhd-6fd9f37       x4xx/uhd-6fd9f37/x4xx_x410_cpld_default-g6fd9f37.zip       74e09bedda84105258484217d4dbf68860ce2079d27cf8352952fe8e4d727e93
+x4xx_zbx_cpld_default           uhd-d5c2750       x4xx/uhd-d5c2750/x4xx_zbx_cpld_default-gd5c2750.zip        f4e2fce5a57052b0efd9b4b247dd816d842de63b03091bd4f030dc01fd5a3fd0
+# X410-Series Filesystems
+x4xx_common_sdk_default         meta-ettus-v4.1.0.5  x4xx/meta-ettus-v4.1.0.5/x4xx_common_sdk_default-v4.1.0.5.zip       0
+x4xx_common_mender_default      meta-ettus-v4.1.0.5  x4xx/meta-ettus-v4.1.0.5/x4xx_common_mender_default-v4.1.0.5.zip    0
+x4xx_common_sdimg_default       meta-ettus-v4.1.0.5  x4xx/meta-ettus-v4.1.0.5/x4xx_common_sdimg_default-v4.1.0.5.zip     0
+
 # X300-Series
-x3xx_x310_fpga_default          fpga-fde2a94eb       x3xx/fpga-fde2a94eb/x3xx_x310_fpga_default-gfde2a94e.zip                 ac5bcf4b85773508bdc3b5e882bfa14025a7867b4c1858f5751439919ee6d270
-x3xx_x300_fpga_default          fpga-fde2a94eb       x3xx/fpga-fde2a94eb/x3xx_x300_fpga_default-gfde2a94e.zip                 2bdfb7851f6b9aebadfe3aa227d93532bd005fcae4b4d88ae74522f903bd4081
+x3xx_x310_fpga_default          uhd-26793b8       x3xx/uhd-26793b8/x3xx_x310_fpga_default-g26793b8.zip                  6c5da030d52f9e1d5dbb89d182d5708b4c0964f6c66c4cd8c5c8c99a872795f2
+x3xx_x300_fpga_default          uhd-26793b8       x3xx/uhd-26793b8/x3xx_x300_fpga_default-g26793b8.zip                  c1f053855f561b5fd4f3b1cf2bc5fc9dc93a6d1df4d874fa1c748f07be6661b2
 # Example daughterboard targets (none currently exist)
 #x3xx_twinrx_cpld_default   example_target
 #dboard_ubx_cpld_default    example_target
 
 # E-Series
-e3xx_e310_sg1_fpga_default      fpga-fde2a94eb       e3xx/fpga-fde2a94eb/e3xx_e310_sg1_fpga_default-gfde2a94e.zip                 ac5f6db65780e944c565977e0bd5085c76d0bcdf246579f13d5da69211590980
-e3xx_e310_sg3_fpga_default      fpga-fde2a94eb       e3xx/fpga-fde2a94eb/e3xx_e310_sg3_fpga_default-gfde2a94e.zip                 84ddae28d98d02cdeb88b560dc6c1f7280f8d9adcc719bec9774deb7ed6dbb3c
-e3xx_e320_fpga_default          fpga-fde2a94eb       e3xx/fpga-fde2a94eb/e3xx_e320_fpga_default-gfde2a94e.zip                 95064badda9b93281b01e642faff9577d054883869896899e28293a243d8862c
+e3xx_e310_sg1_fpga_default      uhd-26793b8       e3xx/uhd-26793b8/e3xx_e310_sg1_fpga_default-g26793b8.zip             5d4e087a2ec732f01da0e260a62a48f11564be63c9b8d6ebd7fd33139c5fff94
+e3xx_e310_sg3_fpga_default      uhd-26793b8       e3xx/uhd-26793b8/e3xx_e310_sg3_fpga_default-g26793b8.zip             a257f345da886239295a2bb9d7c6d1f9064a8431155c28d080924c400ba1d783
+e3xx_e320_fpga_default          uhd-26793b8       e3xx/uhd-26793b8/e3xx_e320_fpga_default-g26793b8.zip                 aac70d50c09cf232dd58e7008e29f6b359ccc7bf8beb4303a95028515dcdd942
 
 # E310 Filesystems
-e3xx_e310_sdk_default         meta-ettus-v3.15.0.0  e3xx/meta-ettus-v3.15.0.0/e3xx_e310_sdk_default-v3.15.0.0.zip       0
-e3xx_e310_sg1_mender_default  meta-ettus-v3.15.0.0  e3xx/meta-ettus-v3.15.0.0/e3xx_e310_sg1_mender_default-v3.15.0.0.zip    0
-e3xx_e310_sg1_sdimg_default   meta-ettus-v3.15.0.0  e3xx/meta-ettus-v3.15.0.0/e3xx_e310_sg1_sdimg_default-v3.15.0.0.zip     0
-e3xx_e310_sg3_mender_default  meta-ettus-v3.15.0.0  e3xx/meta-ettus-v3.15.0.0/e3xx_e310_sg3_mender_default-v3.15.0.0.zip    0
-e3xx_e310_sg3_sdimg_default   meta-ettus-v3.15.0.0  e3xx/meta-ettus-v3.15.0.0/e3xx_e310_sg3_sdimg_default-v3.15.0.0.zip     0
+e3xx_e310_sdk_default         meta-ettus-v4.1.0.5  e3xx/meta-ettus-v4.1.0.5/e3xx_e310_sdk_default-v4.1.0.5.zip       0
+e3xx_e310_sg1_mender_default  meta-ettus-v4.1.0.5  e3xx/meta-ettus-v4.1.0.5/e3xx_e310_sg1_mender_default-v4.1.0.5.zip    0
+e3xx_e310_sg1_sdimg_default   meta-ettus-v4.1.0.5  e3xx/meta-ettus-v4.1.0.5/e3xx_e310_sg1_sdimg_default-v4.1.0.5.zip     0
+e3xx_e310_sg3_mender_default  meta-ettus-v4.1.0.5  e3xx/meta-ettus-v4.1.0.5/e3xx_e310_sg3_mender_default-v4.1.0.5.zip    0
+e3xx_e310_sg3_sdimg_default   meta-ettus-v4.1.0.5  e3xx/meta-ettus-v4.1.0.5/e3xx_e310_sg3_sdimg_default-v4.1.0.5.zip     0
 
 # E320 Filesystems, etc
-e3xx_e320_sdk_default         meta-ettus-v3.15.0.0  e3xx/meta-ettus-v3.15.0.0/e3xx_e320_sdk_default-v3.15.0.0.zip       0
-e3xx_e320_mender_default      meta-ettus-v3.15.0.0  e3xx/meta-ettus-v3.15.0.0/e3xx_e320_mender_default-v3.15.0.0.zip    0
-e3xx_e320_sdimg_default       meta-ettus-v3.15.0.0  e3xx/meta-ettus-v3.15.0.0/e3xx_e320_sdimg_default-v3.15.0.0.zip     0
+e3xx_e320_sdk_default         meta-ettus-v4.1.0.5  e3xx/meta-ettus-v4.1.0.5/e3xx_e320_sdk_default-v4.1.0.5.zip       0
+e3xx_e320_mender_default      meta-ettus-v4.1.0.5  e3xx/meta-ettus-v4.1.0.5/e3xx_e320_mender_default-v4.1.0.5.zip    0
+e3xx_e320_sdimg_default       meta-ettus-v4.1.0.5  e3xx/meta-ettus-v4.1.0.5/e3xx_e320_sdimg_default-v4.1.0.5.zip     0
 
 # N300-Series
-n3xx_n310_fpga_default          fpga-fde2a94eb       n3xx/fpga-fde2a94eb/n3xx_n310_fpga_default-gfde2a94e.zip                 4a45e2d4ec41d34fc550402928248a15488ba7b58d835ef2d4f503556173764a
-n3xx_n300_fpga_default          fpga-fde2a94eb       n3xx/fpga-fde2a94eb/n3xx_n300_fpga_default-gfde2a94e.zip                 7f1b512756ae225f8caaf20fd22d2e1537ea211b2fd6c4c9ed1330065fb56645
-n3xx_n320_fpga_default          fpga-fde2a94eb        n3xx/fpga-fde2a94eb/n3xx_n320_fpga_default-gfde2a94e.zip                4fae9de2ede7ff446da8afb61808868c08ba32f8e3092b802c5d4d7643c7175f
+n3xx_n310_fpga_default          uhd-26793b8       n3xx/uhd-26793b8/n3xx_n310_fpga_default-g26793b8.zip                 c06888dbb7dcc18b2b97eca38bc7f41706806affcc6da50645ec87dd30afd39c
+n3xx_n300_fpga_default          uhd-26793b8       n3xx/uhd-26793b8/n3xx_n300_fpga_default-g26793b8.zip                 946dfa30ac67c016c0dfeb248350202978d8a9325b255911d5211212b19517d2
+n3xx_n320_fpga_default          uhd-26793b8        n3xx/uhd-26793b8/n3xx_n320_fpga_default-g26793b8.zip                64f84f4c3eff2febe0bb5d7ef8ea995dd0f4bb87604c772b6d655dbffa1a9124
 n3xx_n310_cpld_default          fpga-6bea23d        n3xx/fpga-6bea23d/n3xx_n310_cpld_default-g6bea23d.zip                ef128dcd265ee8615b673021d4ee84c39357012ffe8b28c8ad7f893f9dcb94cb
 n3xx_n320_cpld_default          fpga-4bc2c6f       n3xx/fpga-4bc2c6f/n3xx_n320_cpld_default-g4bc2c6f.zip              6680a9363efc5fa8b5a68beb3dff44f2e314b94e716e3a1751aba0fed1f384da
 # N3XX Mykonos firmware
 #n3xx_n310_fw_default           fpga-6bea23d        n3xx/fpga-6bea23d/n3xx_n310_fw_default-g6bea23d.zip                     0
 # N300-Series Filesystems, etc
-n3xx_common_sdk_default         meta-ettus-v3.15.0.0  n3xx/meta-ettus-v3.15.0.0/n3xx_common_sdk_default-v3.15.0.0.zip       0
-n3xx_common_mender_default      meta-ettus-v3.15.0.0  n3xx/meta-ettus-v3.15.0.0/n3xx_common_mender_default-v3.15.0.0.zip    0
-n3xx_common_sdimg_default       meta-ettus-v3.15.0.0  n3xx/meta-ettus-v3.15.0.0/n3xx_common_sdimg_default-v3.15.0.0.zip     0
+n3xx_common_sdk_default         meta-ettus-v4.1.0.5  n3xx/meta-ettus-v4.1.0.5/n3xx_common_sdk_default-v4.1.0.5.zip       0
+n3xx_common_mender_default      meta-ettus-v4.1.0.5  n3xx/meta-ettus-v4.1.0.5/n3xx_common_mender_default-v4.1.0.5.zip    0
+n3xx_common_sdimg_default       meta-ettus-v4.1.0.5  n3xx/meta-ettus-v4.1.0.5/n3xx_common_sdimg_default-v4.1.0.5.zip     0
 
 # B200-Series
-b2xx_b200_fpga_default          fpga-fde2a94eb       b2xx/fpga-fde2a94eb/b2xx_b200_fpga_default-gfde2a94e.zip               44885476f1bce75657e4dd78ba03830e662a59b6bb612a90e9f8041306deb68e
-b2xx_b200mini_fpga_default      fpga-fde2a94eb       b2xx/fpga-fde2a94eb/b2xx_b200mini_fpga_default-gfde2a94e.zip           4318249e394be686e99c58effa162209aa496a039c68ef816f0b2f9247485c12
-b2xx_b210_fpga_default          fpga-fde2a94eb       b2xx/fpga-fde2a94eb/b2xx_b210_fpga_default-gfde2a94e.zip               936864ec5e9627f786c3cd3687c761a24a0ff68a99e224fcc2b24cb4d4197dd2
-b2xx_b205mini_fpga_default      fpga-fde2a94eb       b2xx/fpga-fde2a94eb/b2xx_b205mini_fpga_default-gfde2a94e.zip           cd3dad8c70925c0c79e728716c0ebb36f2145e1cf929409fa77c953637effd70
-b2xx_common_fw_default          uhd-2bdad498         b2xx/uhd-2bdad498/b2xx_common_fw_default-g2bdad498.zip                  a6a867466448f2f75d9d5d290c57ceb8e1d3219391c9f275824fbeb3e7931732
+b2xx_b200_fpga_default          uhd-26793b8       b2xx/uhd-26793b8/b2xx_b200_fpga_default-g26793b8.zip               7fde585623bee8c274cd4db9fb1a3a4738e4ce862dd8bb09a3f1448e37c3cc99
+b2xx_b200mini_fpga_default      uhd-26793b8       b2xx/uhd-26793b8/b2xx_b200mini_fpga_default-g26793b8.zip           0b399b4b4ba9e4039059bc22d7b3ae664e70f1677ca510cbbbbf9d4a1d1bf5ac
+b2xx_b210_fpga_default          uhd-26793b8       b2xx/uhd-26793b8/b2xx_b210_fpga_default-g26793b8.zip               f9ba2406d2bf65aba93f5eb6bb5d8c440aa6ec623117a8b955e5bc58732bd58f
+b2xx_b205mini_fpga_default      uhd-26793b8       b2xx/uhd-26793b8/b2xx_b205mini_fpga_default-g26793b8.zip           21c42ba7342f135db0efa8617edfa0735e72631ce1fb17284940b713ffeb3c4b
+b2xx_common_fw_default          uhd-7f7d016         b2xx/uhd-7f7d016/b2xx_common_fw_default-g7f7d016.zip                  22ac0e54cc90d53d8494d481439447a0c89b9bbaced6fabe24df260eda3ce3a3
 
 # USRP2 Devices
 usrp2_usrp2_fw_default          fpga-6bea23d        usrp2/fpga-6bea23d/usrp2_usrp2_fw_default-g6bea23d.zip                  d523a18318cb6a7637be40484bf03a6f54766410fee2c1a1f72e8971ea9a9cb6
@@ -622,7 +715,7 @@ usrp2_n200_fpga_default         fpga-6bea23d        usrp2/fpga-6bea23d/usrp2_n20
 usrp2_n200_fw_default           fpga-6bea23d        usrp2/fpga-6bea23d/usrp2_n200_fw_default-g6bea23d.zip                   3eee2a6195caafe814912167fccf2dfc369f706446f8ecee36e97d2c0830116f
 usrp2_n210_fpga_default         fpga-6bea23d        usrp2/fpga-6bea23d/usrp2_n210_fpga_default-g6bea23d.zip                 5ce68ac539ee6eeb7d04fb3127c1fabcaff442a8edfaaa2f3746590f9df909bd
 usrp2_n210_fw_default           fpga-6bea23d        usrp2/fpga-6bea23d/usrp2_n210_fw_default-g6bea23d.zip                   3646fcd3fc974d18c621cb10dfe97c4dad6d282036dc63b7379995dfad95fb98
-n230_n230_fpga_default          fpga-fde2a94eb        n230/fpga-fde2a94eb/n230_n230_fpga_default-gfde2a94e.zip                   5436932bc8a459c6e350885b67f907f7c864edd71c0089f6692636428b3c20a9
+n230_n230_fpga_default          fpga-e57dfe0        n230/fpga-e57dfe0/n230_n230_fpga_default-ge57dfe0.zip                   f115b78a1893da0c2a481257351b834721f8a5468dfae6a34e3a68a3bc0065cd
 
 # USRP1 Devices
 usrp1_usrp1_fpga_default        fpga-6bea23d        usrp1/fpga-6bea23d/usrp1_usrp1_fpga_default-g6bea23d.zip                03bf72868c900dd0853bf48e2ede91058d579829b0e70c021e51b0e282d1d5be
